function [hpd,prob] = f_mhpd_besag(theta,clev,disp_flag,vec0)
%
% [hpd,prob] = f_mhpd_besag(theta,clev,disp_flag,vec0)
%
% computes Besag et al. (1995), p. 30, multivariate hpd and pseudo contour prob
% from Held (2004), p. 23
%
% output:
% hpd       rows are variables cols are lower bound and upper bound of hpd
%           for each variable (number of parameters x 2)
% prob      pseudo contour prob of vec0 (see below)
%
% input:
% theta     draws of parameter vector (ndraws x number of parameters) 
% clev      nominal coverage confidence level, eg 0.95 for 95% coverage of
%           the hpd
% vec0      (optional) (1 x number of parameters) parameter vector to estimate pseudo contour prob  
%
% Besag, J. E., Green, P. J., Higdon, D. M., and Mengersen, K. L. (1995), "Bayesian Computation and Stochastic
% Systems", Statistical Science, 10, 3-66.
% Held, L. (2004). Simultaneous posterior probability statements from Monte
% Carlo output. Journal of Computational and Graphical Statistics 13, 20-35.
%
N=size(theta,2);
Nrep=size(theta,1);

% sort draws: theta_sort are equal to individual order statistics of theta
[theta_sort,theta_sort_index]=sort(theta,1,'ascend');
rank_sort=kron([1:Nrep]',ones(1,N));
if strcmp(disp_flag,'on')
    disp('theta,theta_sort,theta_sort_index,rank_sort');
    disp([theta,theta_sort,theta_sort_index,rank_sort]);
end

% theta_rank are equal to individual ranks of theta
theta_rank=nan(size(theta));
for i=1:N
    theta_rank(theta_sort_index(:,i),i)=rank_sort(:,i);
end
if strcmp(disp_flag,'on')
    disp('theta,theta_rank');
    disp([theta,theta_rank]);
end

% define selection set 
set=nan(1,Nrep);
for j=1:Nrep
    set(1,j)=max(Nrep+1-min(theta_rank(j,:)),max(theta_rank(j,:)));
end
[set_sort,set_sort_index]=sort(set,2,'ascend');
if strcmp(disp_flag,'on')
    disp([set;set_sort;set_sort_index;[1:Nrep]]);
end

% k-th order stat
k=round(clev*Nrep);
jstar=set_sort(1,k);
lbj=Nrep+1-jstar;
ubj=jstar;
hpd=theta_sort([lbj,ubj],:)';

% pseudo contour probability for vec0 (similar to pval of H0 hypothesis)
prob=nan;
if nargin>3
    %credible bands for all k
    hpd_mat=nan(N,2,Nrep);
    in_vec=nan(1,Nrep);
    for k=1:Nrep
        jstar=set_sort(1,k);
        lbj=Nrep+1-jstar;
        ubj=jstar;
        hpd=theta_sort([lbj,ubj],:)';
        hpd_mat(:,:,k)=hpd;
        if all(vec0'>hpd(:,1))&&all(vec0'<hpd(:,2))
            in_vec(1,k)=1;
        else
            in_vec(1,k)=0;
        end
        %disp(k);
        %disp([hpd,vec0']);disp(in_vec(1,1:Nrep));
        %pause
    end
    k_min=find(in_vec==1,1,'first');
    if isempty(k_min)
        k_min=Nrep;
    end
    prob=1.0-k_min/Nrep;
end

end
