function [rank_bekker_cond,bekker_cond_fulfilled] = f_becker_cond(lambda)
% function [rank_bekker_cond,bekker_cond_fulfilled] = f_becker_cond(lambda)
%   checks bekker condition for static model: takes (Nxr) loadings matrix, and checks 
%   whether the zeros therein fulfill the bekker condition
%
% references:
%   Bai, J., Wang, P., 2014. Identification theory for high dimensional static and dynamic factor models. Journal of Econometrics 178 (2014), 794804
%   Bekker, P.A., 1986. A note on the identification of restricted factor loading matrices. Psychometrika 51 (4), 607611
%
% examples: 
%   cond fulfilled:     lambda=ones(4,2);lambda(1,2)=0;lambda(4,1)=0;[rank_bekker,bekker_cond_fulfilled]=f_becker_cond(lambda)
%   cond not fulfilled: lambda=ones(4,2);lambda(1,2)=0;lambda(1,1)=0;[rank_bekker,bekker_cond_fulfilled]=f_becker_cond(lambda)
%   cond fulfilled:     lambda=zeros(4,2);lambda(1,2)=1;lambda(2,2)=1;[rank_bekker,bekker_cond_fulfilled]=f_becker_cond(lambda)
%   cond not fulfilled: lambda=zeros(4,2);lambda(1,2)=1;lambda(1,1)=1;[rank_bekker,bekker_cond_fulfilled]=f_becker_cond(lambda)
% 

% factor rank
r=size(lambda,2);

% M
veclam=lambda(:);
index_lamzero=find(veclam==0);
M=zeros(length(index_lamzero),length(veclam));
for i=1:length(index_lamzero)
    M(i,index_lamzero(i))=1.0;
end

% duplication matrix
d=zeros(r*r,r*(r+1)/2);
count=0;
for j=1:r
    d((j-1)*r+j,count+j)=1;
    for i=(j+1):r
      d((j-1)*r+i,count+i)= 1;
      d((i-1)*r+j,count+i)= 1;
    end
    count=count+r-j;
end
dinv=(d'*d)\(d');

% bekker condition
rank_bekker_cond=rank([dinv;M*kron(eye(r),lambda)]);
bekker_cond_fulfilled=rank_bekker_cond==r^2;

end


% function [d,dinv] = f_duplication_matrix(r)
% 
% d=zeros(r*r,r*(r+1)/2);
% count=0;
% for j=1:r
%     d((j-1)*r+j,count+j)=1;
%     for i=(j+1):r
%       d((j-1)*r+i,count+i)= 1;
%       d((i-1)*r+j,count+i)= 1;
%     end
%     count=count+r-j;
% end
% dinv=(d'*d)\(d');
% 
% end

