function [zeroout_flag,prob] = f_mhpd_eucmah(theta,clev,method_str,disp_flag,fig_flag,vec0)
% [zeroout_flag,prob] = f_mhpd_eucmah(theta,clev,method_str,disp_flag,fig_flag,vec0)
%
% computes Hanson and Johnson (2002) multivariate HPD, method 2
% based on euclidean or mahalanobis distance
%
% output:
% prob       pseudo contour prob of vec0 (see below)
%            if prob=0.05, then 5% of the draws have a lower probability than vec0
%            if prob<0.05, then vec0 is outside 95% HPD
% out_flag   is 1, if vec0 is outside 95% HPD
%
% input:
% theta      draws of parameter vector (ndraws x number of parameters) 
% clev       nominal coverage confidence level, eg 0.95
% method_str 'euc' leads to euclidean distances, 'mah' to mahalanobis
%            distances as in Hanson, McMillan (2012)
% vec0       (optional) (1 x number of parameters) parameter vector to 
%            estimate pseudo contour prob  
%
% Hanson, T. and McMillan, G. (2012). Scheffe style simultaneous credible bands for regression 
% surfaces with application to Ache honey gathering, Journal of Data Science, 10, 175-193.
%
% for testing: R=500;res=zeros(R,1);for i=1:R; 
%              [out_index,prob] = f_mscb_euc(randn(1000,2)+0.5,0.95,'mah','off','off',[0,0]); 
%              if out_index==1; res(i,1)=1.0; end; end; mean(res)
%
N=size(theta,2);
Nrep=size(theta,1);

if strcmp(method_str,'euc')
    
    % compute all euclidean distances between draws
    eucdist=pdist(theta,'euclidean');
    eucdist=squareform(eucdist);
    eucdist=eucdist+eye(Nrep)*max(max(eucdist));
    min_eucdist=min(eucdist,[],2);
    clear eucdist
    testdist=min_eucdist;

end

if strcmp(method_str,'mah')
    
    % comp mean variance and mahalanobis distance to mean
    mtheta=mean(theta,1)';
    vtheta=cov(theta);
    [S,V,D]=svd(vtheta);
    invV=diag(diag(V).^-1);invV(isinf(invV))=0.0; % inf is turned into zero, zero variance, means inf inv(variance), but differences will be zero minus zero) does not work with general vector!!!
    invvtheta=(D'\invV)/S;
    mahdist=nan(Nrep,1);
    for i=1:Nrep
        theta_sel=theta(i,:)';
        if cond(vtheta)>1.0e+10
            mahdist(i,1)=((theta_sel-mtheta)'*invvtheta)*(theta_sel-mtheta);
        else
            mahdist(i,1)=((theta_sel-mtheta)'/vtheta)*(theta_sel-mtheta);
        end
    end
    testdist=mahdist;
    
end

% remove draws with highest euclidean/mahalanobis distance
[testdist_sort,testdist_sort_index]=sort(testdist,1,'ascend');
Nkeep=round(Nrep*clev);
[~,rank]=sort(testdist_sort_index,1,'ascend');
testdist_min_index=testdist_sort_index(1:Nkeep,1);
testdist_sort_max=max(testdist(testdist_min_index,:)); %testdist_sort_max; mean(testdist>testdist_sort_max)
if strcmp(disp_flag,'on')
    disp([[1:Nrep]',theta,testdist,rank]);
    disp(testdist_min_index);
end

% check vec0
prob=nan;
if nargin>5
    if cond(vtheta)>1.0e+10
        mahdist_vec0=((vec0'-mtheta)'*invvtheta)*(vec0'-mtheta);
    else
        mahdist_vec0=((vec0'-mtheta)'/vtheta)*(vec0'-mtheta);
    end
    k_min=find(testdist_sort>mahdist_vec0,1,'first');
    if isempty(k_min)
        k_min=Nrep;
    end
    prob=1.0-k_min/Nrep;
    if prob<1.0-clev; out_prob=1.0; else out_prob=0; end
    if mahdist_vec0>testdist_sort_max; out_dist=1.0; else out_dist=0; end
    if strcmp(disp_flag,'on')
        disp(['out_prob ',num2str(out_prob),', out_dist ',num2str(out_dist)]);
    end
    zeroout_flag=out_prob;
end

% show theta
if strcmp(fig_flag,'on')
    testdist_min_index=testdist_sort_index(1:k_min,1);
    theta_scb=theta(testdist_min_index,:);
    if N==2
        figure; newplot;
        scatter(theta(:,1),theta(:,2),5,'MarkerEdgeColor','b');
        hold on
        scatter(theta_scb(:,1),theta_scb(:,2),5,'MarkerEdgeColor','r');
        title(['simulated sample (N=',num2str(N),',Nrep=',num2str(Nrep),')']);
        legend({'full sample',['at point: (',num2str(vec0(1,1)),', ',num2str(vec0(1,2)),')']});
    end
end

end
